@@ -76,13 +76,21 @@ pkg_preinst() {
 	else
 		einfo "has_version does not detect an installed instance of $CATEGORY/$PN:$SLOT"
 	fi
+	if [[ ${EPREFIX} != ${PORTAGE_OVERRIDE_EPREFIX} ]] ; then
+		if has_version --host-root $CATEGORY/$PN:$SLOT ; then
+			einfo "has_version --host-root detects an installed instance of $CATEGORY/$PN:$SLOT"
+			einfo "best_version --host-root reports that the installed instance is $(best_version $CATEGORY/$PN:$SLOT)"
+		else
+			einfo "has_version --host-root does not detect an installed instance of $CATEGORY/$PN:$SLOT"
+		fi
+	fi
 }
 
 """
 
 		ebuilds = {
 			"dev-libs/A-1": {
-				"EAPI" : "4",
+				"EAPI" : "5",
 				"IUSE" : "+flag",
 				"KEYWORDS": "x86",
 				"LICENSE": "GPL-2",
@@ -90,14 +98,14 @@ pkg_preinst() {
 				"RDEPEND": "flag? ( dev-libs/B[flag] )",
 			},
 			"dev-libs/B-1": {
-				"EAPI" : "4",
+				"EAPI" : "5",
 				"IUSE" : "+flag",
 				"KEYWORDS": "x86",
 				"LICENSE": "GPL-2",
 				"MISC_CONTENT": install_something,
 			},
 			"virtual/foo-0": {
-				"EAPI" : "4",
+				"EAPI" : "5",
 				"KEYWORDS": "x86",
 				"LICENSE": "GPL-2",
 			},
@@ -105,7 +113,7 @@ pkg_preinst() {
 
 		installed = {
 			"dev-libs/A-1": {
-				"EAPI" : "4",
+				"EAPI" : "5",
 				"IUSE" : "+flag",
 				"KEYWORDS": "x86",
 				"LICENSE": "GPL-2",
@@ -113,21 +121,21 @@ pkg_preinst() {
 				"USE": "flag",
 			},
 			"dev-libs/B-1": {
-				"EAPI" : "4",
+				"EAPI" : "5",
 				"IUSE" : "+flag",
 				"KEYWORDS": "x86",
 				"LICENSE": "GPL-2",
 				"USE": "flag",
 			},
 			"dev-libs/depclean-me-1": {
-				"EAPI" : "4",
+				"EAPI" : "5",
 				"IUSE" : "",
 				"KEYWORDS": "x86",
 				"LICENSE": "GPL-2",
 				"USE": "",
 			},
 			"app-misc/depclean-me-1": {
-				"EAPI" : "4",
+				"EAPI" : "5",
 				"IUSE" : "",
 				"KEYWORDS": "x86",
 				"LICENSE": "GPL-2",
@@ -160,10 +168,10 @@ pkg_preinst() {
 		eroot = settings["EROOT"]
 		trees = playground.trees
 		portdb = trees[eroot]["porttree"].dbapi
-		portdir = settings["PORTDIR"]
+		test_repo_location = settings.repositories["test_repo"].location
 		var_cache_edb = os.path.join(eprefix, "var", "cache", "edb")
 		cachedir = os.path.join(var_cache_edb, "dep")
-		cachedir_pregen = os.path.join(portdir, "metadata", "md5-cache")
+		cachedir_pregen = os.path.join(test_repo_location, "metadata", "md5-cache")
 
 		portage_python = portage._python_interpreter
 		dispatch_conf_cmd = (portage_python, "-Wd",
@@ -171,7 +179,9 @@ pkg_preinst() {
 		ebuild_cmd = (portage_python, "-Wd",
 			os.path.join(PORTAGE_BIN_PATH, "ebuild"))
 		egencache_cmd = (portage_python, "-Wd",
-			os.path.join(PORTAGE_BIN_PATH, "egencache"))
+			os.path.join(PORTAGE_BIN_PATH, "egencache"),
+			"--repo", "test_repo",
+			"--repositories-configuration", settings.repositories.config_string())
 		emerge_cmd = (portage_python, "-Wd",
 			os.path.join(PORTAGE_BIN_PATH, "emerge"))
 		emaint_cmd = (portage_python, "-Wd",
@@ -201,6 +211,8 @@ pkg_preinst() {
 		test_ebuild = portdb.findname("dev-libs/A-1")
 		self.assertFalse(test_ebuild is None)
 
+		cross_prefix = os.path.join(eprefix, "cross_prefix")
+
 		test_commands = (
 			env_update_cmd,
 			portageq_cmd + ("envvar", "-v", "CONFIG_PROTECT", "EROOT",
@@ -266,6 +278,24 @@ pkg_preinst() {
 			emerge_cmd + ("-p", "--unmerge", "-q", eroot + "usr"),
 			emerge_cmd + ("--unmerge", "--quiet", "dev-libs/A"),
 			emerge_cmd + ("-C", "--quiet", "dev-libs/B"),
+
+			# Test cross-prefix usage, including chpathtool for binpkgs.
+			({"EPREFIX" : cross_prefix},) + \
+				emerge_cmd + ("--usepkgonly", "dev-libs/A"),
+			({"EPREFIX" : cross_prefix},) + \
+				portageq_cmd + ("has_version", cross_prefix, "dev-libs/A"),
+			({"EPREFIX" : cross_prefix},) + \
+				portageq_cmd + ("has_version", cross_prefix, "dev-libs/B"),
+			({"EPREFIX" : cross_prefix},) + \
+				emerge_cmd + ("-C", "--quiet", "dev-libs/B"),
+			({"EPREFIX" : cross_prefix},) + \
+				emerge_cmd + ("-C", "--quiet", "dev-libs/A"),
+			({"EPREFIX" : cross_prefix},) + \
+				emerge_cmd + ("dev-libs/A",),
+			({"EPREFIX" : cross_prefix},) + \
+				portageq_cmd + ("has_version", cross_prefix, "dev-libs/A"),
+			({"EPREFIX" : cross_prefix},) + \
+				portageq_cmd + ("has_version", cross_prefix, "dev-libs/B"),
 		)
 
 		distdir = playground.distdir
@@ -309,6 +339,7 @@ pkg_preinst() {
 			"PORTAGE_INST_GID" : str(portage.data.portage_gid),
 			"PORTAGE_INST_UID" : str(portage.data.portage_uid),
 			"PORTAGE_PYTHON" : portage_python,
+			"PORTAGE_REPOSITORIES" : settings.repositories.config_string(),
 			"PORTAGE_TMPDIR" : portage_tmpdir,
 			"PYTHONPATH" : pythonpath,
 			"__PORTAGE_TEST_PATH_OVERRIDE" : fake_bin,
@@ -318,7 +349,7 @@ pkg_preinst() {
 			env["__PORTAGE_TEST_HARDLINK_LOCKS"] = \
 				os.environ["__PORTAGE_TEST_HARDLINK_LOCKS"]
 
-		updates_dir = os.path.join(portdir, "profiles", "updates")
+		updates_dir = os.path.join(test_repo_location, "profiles", "updates")
 		dirs = [cachedir, cachedir_pregen, distdir, fake_bin,
 			portage_tmpdir, updates_dir,
 			user_config_dir, var_cache_edb]
@@ -344,7 +375,7 @@ pkg_preinst() {
 			with open(os.path.join(profile_path, "packages"), 'w') as f:
 				f.write("*dev-libs/token-system-pkg")
 			for cp, xml_data in metadata_xml_files:
-				with open(os.path.join(portdir, cp, "metadata.xml"), 'w') as f:
+				with open(os.path.join(test_repo_location, cp, "metadata.xml"), 'w') as f:
 					f.write(playground.metadata_xml_template % xml_data)
 			with open(os.path.join(updates_dir, "1Q-2010"), 'w') as f:
 				f.write("""
