@@ -1,4 +1,4 @@
-# Copyright 2010-2012 Gentoo Foundation
+# Copyright 2010-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from itertools import permutations
@@ -59,13 +59,15 @@ class ResolverPlayground(object):
 		user_config={}, sets={}, world=[], world_sets=[], distfiles={},
 		targetroot=False, debug=False):
 		"""
-		ebuilds: cpv -> metadata mapping simulating available ebuilds. 
+		ebuilds: cpv -> metadata mapping simulating available ebuilds.
 		installed: cpv -> metadata mapping simulating installed packages.
 			If a metadata key is missing, it gets a default value.
 		profile: settings defined by the profile.
 		"""
 		self.debug = debug
 		self.eprefix = normalize_path(tempfile.mkdtemp())
+		portage.const.EPREFIX = self.eprefix.rstrip(os.sep)
+
 		self.eroot = self.eprefix + os.sep
 		if targetroot:
 			self.target_root = os.path.join(self.eroot, 'target_root')
@@ -73,15 +75,13 @@ class ResolverPlayground(object):
 			self.target_root = os.sep
 		self.distdir = os.path.join(self.eroot, "var", "portage", "distfiles")
 		self.pkgdir = os.path.join(self.eprefix, "pkgdir")
-		self.portdir = os.path.join(self.eroot, "usr/portage")
 		self.vdbdir = os.path.join(self.eroot, "var/db/pkg")
-		os.makedirs(self.portdir)
 		os.makedirs(self.vdbdir)
 
 		if not debug:
 			portage.util.noiselimit = -2
 
-		self.repo_dirs = {}
+		self._repositories = {}
 		#Make sure the main repo is always created
 		self._get_repo_dir("test_repo")
 
@@ -95,20 +95,19 @@ class ResolverPlayground(object):
 		self.settings, self.trees = self._load_config()
 
 		self._create_ebuild_manifests(ebuilds)
-		
+
 		portage.util.noiselimit = 0
 
 	def _get_repo_dir(self, repo):
 		"""
 		Create the repo directory if needed.
 		"""
-		if repo not in self.repo_dirs:
+		if repo not in self._repositories:
 			if repo == "test_repo":
-				repo_path = self.portdir
-			else:
-				repo_path = os.path.join(self.eroot, "usr", "local", repo)
+				self._repositories["DEFAULT"] = {"main-repo": repo}
 
-			self.repo_dirs[repo] = repo_path
+			repo_path = os.path.join(self.eroot, "var", "repositories", repo)
+			self._repositories[repo] = {"location": repo_path}
 			profile_path = os.path.join(repo_path, "profiles")
 
 			try:
@@ -121,7 +120,7 @@ class ResolverPlayground(object):
 			f.write("%s\n" % repo)
 			f.close()
 
-		return self.repo_dirs[repo]
+		return self._repositories[repo]["location"]
 
 	def _create_distfiles(self, distfiles):
 		os.makedirs(self.distdir)
@@ -253,9 +252,12 @@ class ResolverPlayground(object):
 		except os.error:
 			pass
 
-		for repo in self.repo_dirs:
+		for repo in self._repositories:
+			if repo == "DEFAULT":
+				continue
+
 			repo_dir = self._get_repo_dir(repo)
-			profile_dir = os.path.join(self._get_repo_dir(repo), "profiles")
+			profile_dir = os.path.join(repo_dir, "profiles")
 			metadata_dir = os.path.join(repo_dir, "metadata")
 			os.makedirs(metadata_dir)
 
@@ -273,17 +275,17 @@ class ResolverPlayground(object):
 			for cat in categories:
 				f.write(cat + "\n")
 			f.close()
-			
+
 			#Create $REPO/profiles/license_groups
 			license_file = os.path.join(profile_dir, "license_groups")
 			f = open(license_file, "w")
 			f.write("EULA TEST\n")
 			f.close()
 
-			repo_config = repo_configs.get(repo) 
+			repo_config = repo_configs.get(repo)
 			if repo_config:
 				for config_file, lines in repo_config.items():
-					if config_file not in self.config_files and not any(fnmatch.fnmatch(config_file, os.path.join(x, "*")) for x in self.config_files): 
+					if config_file not in self.config_files and not any(fnmatch.fnmatch(config_file, os.path.join(x, "*")) for x in self.config_files):
 						raise ValueError("Unknown config file: '%s'" % config_file)
 
 					if config_file in ("layout.conf",):
@@ -362,31 +364,14 @@ class ResolverPlayground(object):
 				with open(os.path.join(metadata_dir, "metadata.xml"), 'w') as f:
 					f.write(herds_xml)
 
-		# Write empty entries for each repository, in order to exercise
-		# RepoConfigLoader's repos.conf processing.
-		repos_conf_file = os.path.join(user_config_dir, "repos.conf")		
-		f = open(repos_conf_file, "w")
-		for repo in sorted(self.repo_dirs.keys()):
-			f.write("[%s]\n" % repo)
-			f.write("\n")
-		f.close()
-
-		portdir_overlay = []
-		for repo_name in sorted(self.repo_dirs):
-			path = self.repo_dirs[repo_name]
-			if path != self.portdir:
-				portdir_overlay.append(path)
-
 		make_conf = {
 			"ACCEPT_KEYWORDS": "x86",
 			"CLEAN_DELAY": "0",
 			"DISTDIR" : self.distdir,
 			"EMERGE_WARNING_DELAY": "0",
 			"PKGDIR": self.pkgdir,
-			"PORTDIR": self.portdir,
 			"PORTAGE_INST_GID": str(portage.data.portage_gid),
 			"PORTAGE_INST_UID": str(portage.data.portage_uid),
-			"PORTDIR_OVERLAY": " ".join("'%s'" % x for x in portdir_overlay),
 			"PORTAGE_TMPDIR": os.path.join(self.eroot, 'var/tmp'),
 		}
 
@@ -410,7 +395,7 @@ class ResolverPlayground(object):
 		if not portage.process.sandbox_capable or \
 			os.environ.get("SANDBOX_ON") == "1":
 			# avoid problems from nested sandbox instances
-			make_conf_lines.append('FEATURES="${FEATURES} -sandbox"')
+			make_conf_lines.append('FEATURES="${FEATURES} -sandbox -usersandbox"')
 
 		configs = user_config.copy()
 		configs["make.conf"] = make_conf_lines
@@ -434,7 +419,7 @@ class ResolverPlayground(object):
 
 		#Create /usr/share/portage/config/sets/portage.conf
 		default_sets_conf_dir = os.path.join(self.eroot, "usr/share/portage/config/sets")
-		
+
 		try:
 			os.makedirs(default_sets_conf_dir)
 		except os.error:
@@ -482,7 +467,11 @@ class ResolverPlayground(object):
 		if self.target_root != os.sep:
 			create_trees_kwargs["target_root"] = self.target_root
 
-		trees = portage.create_trees(env={}, eprefix=self.eprefix,
+		env = {
+			"PORTAGE_REPOSITORIES": "\n".join("[%s]\n%s" % (repo_name, "\n".join("%s = %s" % (k, v) for k, v in repo_config.items())) for repo_name, repo_config in self._repositories.items())
+		}
+
+		trees = portage.create_trees(env=env, eprefix=self.eprefix,
 			**create_trees_kwargs)
 
 		for root, root_trees in trees.items():
@@ -520,7 +509,7 @@ class ResolverPlayground(object):
 				rval, cleanlist, ordered, req_pkg_count = \
 					calc_depclean(self.settings, self.trees, None,
 					options, action, InternalPackageSet(initial_atoms=atoms, allow_wildcard=True), None)
-				result = ResolverPlaygroundDepcleanResult( \
+				result = ResolverPlaygroundDepcleanResult(
 					atoms, rval, cleanlist, ordered, req_pkg_count)
 			else:
 				params = create_depgraph_params(options, action)
@@ -547,7 +536,6 @@ class ResolverPlayground(object):
 		for eroot in self.trees:
 			portdb = self.trees[eroot]["porttree"].dbapi
 			portdb.close_caches()
-			portage.dbapi.porttree.portdbapi.portdbapi_instances.remove(portdb)
 		if self.debug:
 			print("\nEROOT=%s" % self.eroot)
 		else:
@@ -764,7 +752,7 @@ class ResolverPlaygroundResult(object):
 		if self.depgraph._dynamic_config._circular_dependency_handler is not None:
 			handler = self.depgraph._dynamic_config._circular_dependency_handler
 			sol = handler.solutions
-			self.circular_dependency_solutions = dict( zip([x.cpv for x in sol.keys()], sol.values()) )
+			self.circular_dependency_solutions = dict(zip([x.cpv for x in sol.keys()], sol.values()))
 
 class ResolverPlaygroundDepcleanResult(object):
 
